#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package event;
use strict;
no strict 'refs';
use threads;
use threads::shared;

our %hdlr : shared;

my @timers : shared;

use constant {
	PRECISION => 1,
};

our $SUCCESS = -1;
our $DONTCARE = -2;
our $EAGAIN = -3;
our $ENOTIMPL = -4; # like ENOSYS

sub addhandler {
	my ($event, $src, $dst, $owner, $callback) = @_;
	$src = lc $src; $dst = lc $dst;

	unless($event::hdlr{$event}) { $event::hdlr{$event} = &share([]); }
	#push @{$event::hdlr{$event}}, [$src, $dst, $owner, $cbr];
	my $len = @{$event::hdlr{$event}};
	$event::hdlr{$event}[$len] = &share([]);
	$event::hdlr{$event}[$len][0] = $src;
	$event::hdlr{$event}[$len][1] = $dst;
	$event::hdlr{$event}[$len][2] = $owner;
	$event::hdlr{$event}[$len][3] = $callback;
}

sub addtimer($$$$) {
	my ($timercookie, $delta, $owner, $callback) = @_;
	#net::privmsg($config::conf{local}, $config::conf{diag}, "adding timer: ".$timercookie."\n");
	$delta /= PRECISION();
	$delta--;
	
	unless(defined($timers[$delta])) {
		$timers[$delta] = &share([]);
	}
	my $len = @{$timers[$delta]};
	#share($timers[$delta][$len]);
	$timers[$delta][$len] = &share([]);
	$timers[$delta][$len][0] = $timercookie;
	$timers[$delta][$len][1] = $owner;
	$timers[$delta][$len][2] = $callback;
	#net::privmsg($config::conf{local}, $config::conf{diag}, "Timer added:" . join(' ', @{$timers[$delta][$len]}));
}

sub checktimers() {
	module::begin();

	while(1) {
		{
			lock $thread::timercond;
			cond_timedwait($thread::timercond, time() + PRECISION());
		}
		last unless thread::alive();
		
		my $timerlist = shift @timers;
	
		unless(defined($timerlist)) { 
			next;
		}
	
		foreach my $timer (@$timerlist) {
			eval { &{$timer->[2]}($timer->[0]); };
		}
	}
	
	module::end();
}

1;
